@@ -25,7 +25,7 @@ static const struct configdef_kv def_general[] = {
     { CFG_GNRL_MODE,      CFG_VIEWER  },
     { CFG_GNRL_SIZE,      "1280,720"  },
     { CFG_GNRL_POSITION,  CFG_AUTO    },
-    { CFG_GNRL_OVERLAY,   CFG_YES     },
+    { CFG_GNRL_OVERLAY,   CFG_NO      },
     { CFG_GNRL_DECOR,     CFG_NO      },
     { CFG_GNRL_SIGUSR1,   "reload"    },
     { CFG_GNRL_SIGUSR2,   "next_file" },
@@ -33,6 +33,12 @@ static const struct configdef_kv def_general[] = {
     { CFG_GNRL_APP_ID,    "swayimg"   },
 };
 
+static const struct configdef_kv def_drm[] = {
+    { CFG_DRM_PATH,       CFG_AUTO    },
+    { CFG_DRM_CONNECTOR,  CFG_AUTO    },
+    { CFG_DRM_MODE,       CFG_AUTO    },
+};
+
 static const struct configdef_kv def_viewer[] = {
     { CFG_VIEW_WINDOW,    "#00000000" },
     { CFG_VIEW_TRANSP,    "grid"      },
@@ -86,6 +92,7 @@ static const struct configdef_kv def_font[] = {
 
 static const struct configdef_kv def_info[] = {
     { CFG_INFO_SHOW,      CFG_YES     },
+    { CFG_INFO_PADDING,   "10"        },
     { CFG_INFO_ITIMEOUT,  "5"         },
     { CFG_INFO_STIMEOUT,  "3"         },
 };
@@ -143,6 +150,7 @@ static const struct configdef_kv def_keysv[] = {
     { "BackSpace",        "zoom optimal"                             },
     { "k",                "zoom keep"                                },
     { "Alt+s",            "zoom"                                     },
+    { "Alt+p",            "position"                                 },
     { "bracketleft",      "rotate_left"                              },
     { "bracketright",     "rotate_right"                             },
     { "m",                "flip_vertical"                            },
@@ -157,8 +165,8 @@ static const struct configdef_kv def_keysv[] = {
     { "ScrollRight",      "step_left 5"                              },
     { "ScrollUp",         "step_up 5"                                },
     { "ScrollDown",       "step_down 5"                              },
-    { "Ctrl+ScrollUp",    "zoom +10"                                 },
-    { "Ctrl+ScrollDown",  "zoom -10"                                 },
+    { "Ctrl+ScrollUp",    "zoom +10 mouse"                           },
+    { "Ctrl+ScrollDown",  "zoom -10 mouse"                           },
     { "Shift+ScrollUp",   "prev_file"                                },
     { "Shift+ScrollDown", "next_file"                                },
     { "Alt+ScrollUp",     "prev_frame"                               },
@@ -229,6 +237,7 @@ struct configdef_section {
 
 static const struct configdef_section default_config[] = {
     CONFIGDEF_SECTION(CFG_GENERAL, def_general),
+    CONFIGDEF_SECTION(CFG_DRM, def_drm),
     CONFIGDEF_SECTION(CFG_VIEWER, def_viewer),
     CONFIGDEF_SECTION(CFG_SLIDESHOW, def_slideshow),
     CONFIGDEF_SECTION(CFG_GALLERY, def_gallery),
@@ -289,6 +298,9 @@ static bool load(struct config* cfg, const char* path)
         while (nread-- && isspace(line[nread])) {
             line[nread] = 0;
         }
+
+        // false-positive assert for `isspace`
+        // NOLINTNEXTLINE(clang-analyzer-security.ArrayBound)
         while (*line && isspace(*line)) {
             ++line;
         }
@@ -346,11 +358,15 @@ static bool load(struct config* cfg, const char* path)
 
         // trim spaces from start of value
         value = delim + 1;
+        // false-positive assert for `isspace`
+        // NOLINTNEXTLINE(clang-analyzer-security.ArrayBound)
         while (*value && isspace(*value)) {
             ++value;
         }
         // trim spaces from key
         *delim = 0;
+        // false-positive assert for `isspace`
+        // NOLINTNEXTLINE(clang-analyzer-security.ArrayBound)
         while (line != delim && isspace(*--delim)) {
             *delim = 0;
         }
@@ -426,7 +442,7 @@ static bool text_to_color(const char* text, argb_t* value)
     char* endptr;
     argb_t color;
 
-    while (*text == '#' || isspace(*text)) {
+    while (*text && (*text == '#' || isspace(*text))) {
         ++text;
     }
 
